//Defines & Includes
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <inttypes.h>
#include <stdlib.h>
#include <util/delay.h>

#include "config.h"
#include "rs232.h"
#include "lib_io.h"
#include "twi_slave.h"
#include "adc.h"

volatile uint8_t  scan_flag = 0;
volatile uint8_t	read_byte[10];
volatile uint8_t	send_byte[9];
uint8_t init;

ISR(TIMER0_OVF_vect)
{
	if (scan_flag == 0)
	{
		scan_flag = 1;			//alle 18,5 ms
	}
}


ISR (TWI_vect)
{
	uint8_t	i=0;
	//uint8_t	read_byte[10];
	//uint8_t	send_byte[9];
	uint8_t	TWIS_ResonseType;
	
	//LED fr I2C-Communikation einschalten
	PORTB |= (1<<O_LED_01);
	
	send_byte[0] = (PINC & (1<<PINC2));
	send_byte[1] = (PINC & (1<<PINC3));
	send_byte[2] = (PINC & (1<<PINC4));
	send_byte[3] = (PINC & (1<<PINC5));
	send_byte[4] = (PINC & (1<<PINC6));
	send_byte[5] = (PINC & (1<<PINC7));
	send_byte[6] = (PINB & (1<<PINB0));
	send_byte[7] = (PINB & (1<<PINB1));
	//send_byte[8] = (Read_Channel() * ADC_CALIBRATION)/1000;
	send_byte[8] = 200;

	if (TWIS_ResonseRequired (&TWIS_ResonseType))
	{
		switch (TWIS_ResonseType)
		{
			//Daten vom Master empfangen (10 Byte)
			case TWIS_ReadBytes:
				for (i=0; i<9; i++)
				{
					read_byte[i] = TWIS_ReadAck ();
				}
				read_byte[9] = TWIS_ReadNack ();
				
				#if DEBUG_ON
					uart_puts("i2c_read: ok\r\n");
				#endif
				
				TWIS_Stop ();
				
				if (read_byte[0] == 1) {PORTC = PINC ^ (1<<O_CH_00);}
				if (read_byte[1] == 1) {PORTC = PINC ^ (1<<O_CH_01);}
				if (read_byte[2] == 1) {PORTC = PINC ^ (1<<O_CH_02);}
				if (read_byte[3] == 1) {PORTC = PINC ^ (1<<O_CH_03);}
				if (read_byte[4] == 1) {PORTC = PINC ^ (1<<O_CH_04);}
				if (read_byte[5] == 1) {PORTC = PINC ^ (1<<O_CH_05);}
				if (read_byte[6] == 1) {PORTB = PINB ^ (1<<O_CH_06);}
				if (read_byte[7] == 1) {PORTB = PINB ^ (1<<O_CH_07);}
				OCR1A = read_byte[8];
				if (read_byte[9] == 1) {RESET();}
				
			break;
			
			//Daten an den Master senden (9 Byte)
			case TWIS_WriteBytes:
				for (i=0; i<9; i++)
				{                   
					TWIS_Write (send_byte[i]);
					
					#if DEBUG_ON
						uart_puts("i2c_send: ok\r\n");
					#endif
				}
				TWIS_Stop ();
			break;
		}
	}
	
	//LED fr I2C-Communikation ausschalten
	PORTB &= ~(1<<O_LED_01);
}


int main(void)

{
	DDRA &= ~(1<<S_CH_00) | (1<<S_CH_01) | (1<<S_CH_02) | (1<<S_CH_03) | (1<<S_CH_04) | (1<<S_CH_05) | (1<<S_CH_06);
	PORTA &= ~(1<<S_CH_00) | (1<<S_CH_01) | (1<<S_CH_02) | (1<<S_CH_03) | (1<<S_CH_04) | (1<<S_CH_05) | (1<<S_CH_06);
	DDRD &= ~(1<<S_CH_07);
	PORTD &= ~(1<<S_CH_07);
	
	DDRC |= (1<<O_CH_00) | (1<<O_CH_01) |(1<<O_CH_02) |(1<<O_CH_03) |(1<<O_CH_04) |(1<<O_CH_05);
	PORTC &= ~ (1<<O_CH_00) | (1<<O_CH_01) |(1<<O_CH_02) |(1<<O_CH_03) |(1<<O_CH_04) |(1<<O_CH_05);
	DDRB |= (1<<O_CH_06) | (1<<O_CH_07) |(1<<O_LED_00) |(1<<O_LED_01);
	PORTB &= ~ (1<<O_CH_06) | (1<<O_CH_07) |(1<<O_LED_00) |(1<<O_LED_01);
	
	//DDRD |= (1<<O_LED_02);
	//PORTD &= ~(1<<O_LED_02);
	
	PWM_DDR |= (1<<PWM_PIN);
	PWM_PORT &= ~(1<<PWM_PIN);
	
	#if DEBUG_ON
		uart_init();
	#endif
	
	TCCR0 |= (1<<CS00) | (1<<CS02);
	TIMSK = 1<<TOIE0;
	
	TCCR1A = (1<<WGM10) | (1<<COM1A1);		//PWM, Phase Correct, 8-bit, nicht invertiert
	TCCR1B = (1<<CS10);						//ohne Vorteiler
	
	#if DEBUG_ON
		uart_puts("avr_init: succesfull\r\n");
		uart_puts("avr_run : ok\r\n");
	#endif
	
	//Start TWI Slave with address 15 and bitrate of 100000 Hz
	TWIS_Init(I2C_SLAVE_ADRESS, I2C_BITRATE);
	
	ADC_Init();
	
	#if DEBUG_ON
		uart_puts("i2c_init: ok\r\n");
	#endif
	
	for (init=0; init<9; init++)
	{
		read_byte[init] = 0;
	}
	
	sei();
	
	_delay_ms (500);
	
	//LED fr System Ready einschalten
	PORTB |= (1<<O_LED_00);
	
	/*
	uint8_t	i=0;
	//uint8_t	read_byte[10];
	//uint8_t	send_byte[9];
	uint8_t	TWIS_ResonseType;
	uint8_t	tmp_sreg;
	
	//LED fr I2C-Communikation einschalten
	//PORTB |= (1<<O_LED_01);
	
	send_byte[0] = (PINC & (1<<PINC2));
	send_byte[1] = (PINC & (1<<PINC3));
	send_byte[2] = (PINC & (1<<PINC4));
	send_byte[3] = (PINC & (1<<PINC5));
	send_byte[4] = (PINC & (1<<PINC6));
	send_byte[5] = (PINC & (1<<PINC7));
	send_byte[6] = (PINB & (1<<PINB0));
	send_byte[7] = (PINB & (1<<PINB1));
	//send_byte[8] = (Read_Channel() * ADC_CALIBRATION)/1000;
	//send_byte[8] = 200;
	*/
	
	while(1)
	{
		if (scan_flag == 1)
		{
			taster();
			scan_flag = 0;
			
			//debug
			PORTB ^= (1<<O_LED_00);
		}
		
	/*	
	if (TWIS_ResonseRequired (&TWIS_ResonseType))
	{
		PORTB |= (1<<O_LED_01);

		switch (TWIS_ResonseType)
		{
			//Daten vom Master empfangen (10 Byte)
			case TWIS_ReadBytes:
				for (i=0; i<9; i++)
				{
					read_byte[i] = TWIS_ReadAck ();
				}
				read_byte[9] = TWIS_ReadNack ();
				
				#if DEBUG_ON
					uart_puts("i2c_read: ok\r\n");
				#endif
				
				TWIS_Stop ();
				
				if (read_byte[0] == 1) {PORTC = PINC ^ (1<<O_CH_00);}
				if (read_byte[1] == 1) {PORTC = PINC ^ (1<<O_CH_01);}
				if (read_byte[2] == 1) {PORTC = PINC ^ (1<<O_CH_02);}
				if (read_byte[3] == 1) {PORTC = PINC ^ (1<<O_CH_03);}
				if (read_byte[4] == 1) {PORTC = PINC ^ (1<<O_CH_04);}
				if (read_byte[5] == 1) {PORTC = PINC ^ (1<<O_CH_05);}
				if (read_byte[6] == 1) {PORTB = PINB ^ (1<<O_CH_06);}
				if (read_byte[7] == 1) {PORTB = PINB ^ (1<<O_CH_07);}
				OCR1A = read_byte[8];
				if (read_byte[9] == 1) {RESET();}
				
			break;
			
			//Daten an den Master senden (9 Byte)
			case TWIS_WriteBytes:
				for (i=0; i<9; i++)
				{                   
					TWIS_Write (send_byte[i]);
					
					#if DEBUG_ON
						uart_puts("i2c_send: ok\r\n");
					#endif
				}
				TWIS_Stop ();
			break;
		}
		PORTB &= ~(1<<O_LED_01);

	}
	*/	
		
	}
	
	return 0;
}
